@@ -10,24 +10,13 @@ License: BSD
 """
 
 import datetime
+import operator
 
 from trac.attachment import Attachment
 from trac.resource import Resource
 from trac.search import search_to_sql
 from trac.util.datefmt import to_datetime, to_timestamp, utc
 
-try:
-    from trac.util.compat import itemgetter
-    from trac.util.compat import sorted, set
-except ImportError:
-    # 0.12 compat - sorted and set should already be part of Python 2.4
-    from operator import itemgetter
-
-
-__all__ = ['BlogComment', 'BlogPost',
-           'search_blog_posts', 'search_blog_comments',
-           'get_blog_posts', 'get_blog_comments',
-           'group_posts_by_month', 'get_blog_resources']
 
 # Public functions
 
@@ -36,33 +25,22 @@ def search_blog_posts(env, terms):
     Input is a list of terms.
     Returns a list of tuples with:
         (name, version, publish_time, author, title, body) """
-    assert terms
-    # construct sql + args
-    columns = ['bp1.name', 'bp1.title', 'bp1.body',
-               'bp1.author', 'bp1.categories']
-    if hasattr(env, 'db_query'):
-        with env.db_query as db:
-            search_clause, args = search_to_sql(db, columns, terms)
-    else:
-        db = env.get_db_cnx()
+    columns = ['bp1.name', 'bp1.title', 'bp1.body', 'bp1.author',
+               'bp1.categories']
+    with env.db_query as db:
         search_clause, args = search_to_sql(db, columns, terms)
-    sql = "SELECT bp1.name, bp1.version, bp1.publish_time, bp1.author, " \
-               "bp1.title, bp1.body " \
-               "FROM fullblog_posts bp1," \
-               "(SELECT name, max(version) AS ver " \
-               "FROM fullblog_posts GROUP BY name) bp2 " \
-               "WHERE bp1.version = bp2.ver AND bp1.name = bp2.name " \
-               "AND " + search_clause
-    # perform search
-    if hasattr(env, 'db_query'):
-        cursor = env.db_query(sql, args)
-    else:
-        db = env.get_db_cnx()
-        cursor = db.cursor()
-        cursor.execute(sql, args)
-    # return found items
-    return [(row[0], row[1], to_datetime(row[2], utc), row[3],
-            row[4], row[5]) for row in cursor]
+        return [(row[0], row[1], to_datetime(row[2], utc), row[3],
+                 row[4], row[5]) for row in db(
+                 """
+                 SELECT bp1.name, bp1.version, bp1.publish_time, bp1.author,
+                  bp1.title, bp1.body
+                 FROM fullblog_posts bp1,
+                  (SELECT name, max(version) AS ver
+                   FROM fullblog_posts GROUP BY name) bp2
+                 WHERE bp1.version = bp2.ver AND bp1.name = bp2.name
+                 AND %s
+                 """ % search_clause, args)]
+
 
 def search_blog_comments(env, terms):
     """ Free text search for content of blog posts.
@@ -70,26 +48,15 @@ def search_blog_comments(env, terms):
     Returns a list of tuples with:
         (post_name, comment_number, comment, comment_author, comment_time) """
     assert terms
-    # construct sql + args
     columns = ['author', 'comment']
-    if hasattr(env, 'db_query'):
-        with env.db_query as db:
-            search_clause, args = search_to_sql(db, columns, terms)
-    else:
-        db = env.get_db_cnx()
+    with env.db_query as db:
         search_clause, args = search_to_sql(db, columns, terms)
-    sql = "SELECT name, number, comment, author, time " \
-          "FROM fullblog_comments WHERE " + search_clause
-    # perform search
-    if hasattr(env, 'db_query'):
-        cursor = env.db_query(sql, args)
-    else:
-        db = env.get_db_cnx()
-        cursor = db.cursor()
-        cursor.execute(sql, args)
-    # return found items
-    return [(row[0], row[1], row[2], row[3], to_datetime(row[4], utc))
-            for row in cursor]
+        return [(row[0], row[1], row[2], row[3], to_datetime(row[4], utc))
+                for row in db("""
+                    SELECT name, number, comment, author, time
+                    FROM fullblog_comments WHERE %s
+                    """ % search_clause, args)]
+
 
 def get_blog_posts(env, category='', author='', from_dt=None, to_dt=None,
         all_versions=False):
@@ -108,15 +75,6 @@ def get_blog_posts(env, category='', author='', from_dt=None, to_dt=None,
     Returns a list of tuples of the form:
         (name, version, time, author, title, body, category_list)
     Use 'name' and 'version' to instantiate BlogPost objects."""
-
-    # Get db.like() text for reuse
-    if hasattr(env, 'db_query'):
-        with env.db_query as db:
-            db_like = db.like()
-    else:
-        db = env.get_db_cnx()
-        db_like = db.like()
-
     # Build SQL with list of WHERE restrictions
     time_field = 'bp1.publish_time'
     join_operation = ",(SELECT name, max(version) AS ver " \
@@ -127,43 +85,36 @@ def get_blog_posts(env, category='', author='', from_dt=None, to_dt=None,
     if all_versions:
         time_field = 'bp1.version_time'
         join_operation = ""
-    clauses = [category and ("bp1.categories "+db_like, "%"+category+"%"),
-            author and ("bp1.author=%s", author) or None,
-            from_dt and (time_field+">%s", to_timestamp(from_dt)) or None,
-            to_dt and (time_field+"<%s", to_timestamp(to_dt)) or None]
-    clauses = [arg for arg in clauses if arg]  # Ignore the None values
-    if clauses:
-        where_start = "AND "
-        if not join_operation:
-            where_start = "WHERE "
-        where_clause = where_start + " AND ".join([arg[0] for arg in clauses])
-        args = tuple([arg[1] for arg in clauses])
-    sql = "SELECT bp1.name, bp1.version, bp1.publish_time, bp1.author, " \
-               "bp1.title, bp1.body, bp1.categories " \
-               "FROM fullblog_posts bp1 " \
-               + join_operation + where_clause \
-               + " ORDER BY bp1.publish_time DESC"
+    with env.db_query as db:
+        clauses = [category and ("bp1.categories "+ db.like(), "%"+category+"%"),
+                   author and ("bp1.author=%s", author) or None,
+                   from_dt and (time_field+">%s", to_timestamp(from_dt)) or None,
+                   to_dt and (time_field+"<%s", to_timestamp(to_dt)) or None]
+        clauses = [arg for arg in clauses if arg]  # Ignore the None values
+        if clauses:
+            where_start = "AND "
+            if not join_operation:
+                where_start = "WHERE "
+            where_clause = where_start + " AND ".join([arg[0] for arg in clauses])
+            args = tuple([arg[1] for arg in clauses])
 
-    # Execute SQL
-    if hasattr(env, 'db_query'):
-        with env.db_query as db:
-            cursor = env.db_query(sql, args)
-    else:
-        db = env.get_db_cnx()
-        cursor = db.cursor()
-        cursor.execute(sql, args)
+        # Return the rows
+        blog_posts = []
+        for row in db("""
+                SELECT bp1.name, bp1.version, bp1.publish_time, bp1.author,
+                 bp1.title, bp1.body, bp1.categories
+                FROM fullblog_posts bp1 %s %s
+                ORDER BY bp1.publish_time DESC
+                """ % (join_operation, where_clause), args):
+            # Extra check needed to weed out almost-matches where requested
+            # category is a substring of another (searched using LIKE)
+            categories = _parse_categories(row[6])
+            if category and category not in categories:
+                continue
+            blog_posts.append((row[0], row[1], to_datetime(row[2], utc), row[3],
+                    row[4], row[5], categories))
+        return blog_posts
 
-    # Return the rows
-    blog_posts = []
-    for row in cursor:
-        # Extra check needed to weed out almost-matches where requested
-        # category is a substring of another (searched using LIKE)
-        categories = _parse_categories(row[6])
-        if category and category not in categories:
-            continue
-        blog_posts.append((row[0], row[1], to_datetime(row[2], utc), row[3],
-                row[4], row[5], categories))
-    return blog_posts
 
 def get_blog_comments(env, post_name='', from_dt=None, to_dt=None):
     """ Returns comments as a list of tuples from search based on
@@ -177,44 +128,35 @@ def get_blog_comments(env, post_name='', from_dt=None, to_dt=None):
 
     # Build the list of WHERE restrictions
     clauses = [post_name and ("name=%s", post_name) or None,
-            from_dt and ("time>%s", to_timestamp(from_dt)) or None,
-            to_dt and ("time<%s", to_timestamp(to_dt)) or None]
+               from_dt and ("time>%s", to_timestamp(from_dt)) or None,
+               to_dt and ("time<%s", to_timestamp(to_dt)) or None]
     clauses = [arg for arg in clauses if arg]
     where_clause = ""
     args = None
     if clauses:
         where_clause = "WHERE " + " AND ".join([arg[0] for arg in clauses])
         args = tuple([arg[1] for arg in clauses])
-    sql = "SELECT name, number, comment, author, time " \
-            "FROM fullblog_comments " + where_clause
-
-    # Do the SELECT
-    if hasattr(env, 'db_query'):
-        cursor = env.db_query(sql, args)
-    else:
-        db = env.get_db_cnx()
-        cursor = db.cursor()
-        cursor.execute(sql, args)
 
     # Return the items we have found
     return [(row[0], row[1], row[2], row[3], to_datetime(row[4], utc))
-            for row in cursor]
+            for row in env.db_query("""
+                SELECT name, number, comment, author, time
+                FROM fullblog_comments
+                """ + where_clause, args)]
+
 
 def get_blog_resources(env):
     """ Returns a list of resource instances of existing blog posts (current
     version). The list is ordered by publish_time (newest first). """
-    sql = "SELECT bp1.name FROM fullblog_posts bp1, " \
-          "(SELECT name, max(version) AS ver FROM fullblog_posts " \
-          "GROUP BY name) bp2 WHERE bp1.name = bp2.name AND " \
-          "bp1.version = ver ORDER BY bp1.publish_time DESC"
-    if hasattr(env, 'db_query'):
-        cursor = env.db_query(sql)
-    else:
-        db = env.get_db_cnx()
-        cursor = db.cursor()
-        cursor.execute(sql)
     blog_realm = Resource('blog')
-    return [blog_realm(id=post[0], version=0) for post in cursor]
+    return [blog_realm(id=post, version=0)
+            for post, in env.db_query("""
+                SELECT bp1.name FROM fullblog_posts bp1,
+                 (SELECT name, max(version) AS ver
+                 FROM fullblog_posts GROUP BY name) bp2
+                WHERE bp1.name = bp2.name AND bp1.version = ver
+                ORDER BY bp1.publish_time DESC
+                """)]
 
 # Utility functions
 
@@ -248,7 +190,6 @@ def group_posts_by_month(posts):
             grouped_list.append((current_period, posts_per_month))
     return grouped_list
 
-# Internal functions
 
 def _parse_categories(categories, sep=' '):
     """ Parses the string containing categories separated by sep.
@@ -258,7 +199,6 @@ def _parse_categories(categories, sep=' '):
     # Return the list, leaving out any empty items from split()
     return [category for category in categories.split(sep) if category]
 
-# Classes
 
 class BlogComment(object):
     """ Model class representing a comment on a given post.
@@ -275,6 +215,7 @@ class BlogComment(object):
         """ Requires a name for the blog post that the comment belongs to.
         If no comment_id is passed, it is assumed to not exist. """
         self.env = env
+        self.log = env.log
         self.post_name = post_name
         if number:
             self._load_comment(number)
@@ -310,73 +251,48 @@ class BlogComment(object):
         if warnings or verify_only:
             return warnings
         # No problems (we think), try to save.
-        self.env.log.debug("Creating blog comment number %d for %r" % (
-                number, self.post_name))
-        sql = "INSERT INTO fullblog_comments VALUES (%s, %s, %s, %s, %s)"
-        args = (self.post_name, number, comment, author,
-                to_timestamp(self.time))
-        if hasattr(self.env, 'db_transaction'):
-            self.env.db_transaction(sql, args)
-        else:
-            db = self.env.get_db_cnx()
-            cursor = db.cursor()
-            cursor.execute(sql, args)
-            db.commit()
+        self.log.debug("Creating blog comment number %d for %r",
+                       number, self.post_name)
+        self.env.db_transaction("""
+            INSERT INTO fullblog_comments VALUES (%s,%s,%s,%s,%s)
+            """, (self.post_name, number, comment, author,
+                  to_timestamp(self.time)))
         self._load_comment(number)
         return warnings
 
     def delete(self):
         if not self.post_name or not self.number:
             return False
-        self.env.log.debug("Deleting blog comment number %d for %r" % (
-                self.number, self.post_name))
-        sql = "DELETE FROM fullblog_comments WHERE name=%s AND number=%s"
-        args = (self.post_name, self.number)
-        if hasattr(self.env, 'db_transaction'):
-            self.env.db_transaction(sql, args)
-        else:
-            db = self.env.get_db_cnx()
-            cursor = db.cursor()
-            cursor.execute(sql, args)
-            db.commit()
+        self.log.debug("Deleting blog comment number %d for %r",
+                       self.number, self.post_name)
+        self.env.db_transaction("""
+            DELETE FROM fullblog_comments WHERE name=%s AND number=%s
+            """, (self.post_name, self.number))
         return True
 
     # Internal methods
 
     def _load_comment(self, number):
         """ Loads a comment from database if found. """
-        self.env.log.debug("Fetching blog comment number %d for %r" % (
-                number, self.post_name))
-        sql = "SELECT comment, author, time " \
-                "FROM fullblog_comments " \
-                "WHERE name=%s AND number=%s"
-        args = (self.post_name, number)
-        if hasattr(self.env, 'db_query'):
-            cursor = self.env.db_query(sql, args)
-        else:
-            db = self.env.get_db_cnx()
-            cursor = db.cursor()
-            cursor.execute(sql, args)
-        for row in cursor:
+        self.log.debug("Fetching blog comment number %d for %r",
+                       number, self.post_name)
+        for comment, author, time in self.env.db_query("""
+                SELECT comment, author, time FROM fullblog_comments
+                WHERE name=%s AND number=%s
+                """, (self.post_name, number)):
             self.number = number
-            self.comment = row[0]
-            self.author = row[1]
-            self.time = to_datetime(row[2], utc)
+            self.comment = comment
+            self.author = author
+            self.time = to_datetime(time, utc)
             return True
         return False
 
     def _next_comment_number(self):
         """ Function that returns the next available comment number.
         If no blog post exists (can't attach comment), it returns 0. """
-        sql = "SELECT number FROM fullblog_comments WHERE name=%s"
-        args = (self.post_name,)
-        if hasattr(self.env, 'db_query'):
-            cursor = self.env.db_query(sql, args)
-        else:
-            db = self.env.get_db_cnx()
-            cursor = db.cursor()
-            cursor.execute(sql, args)
-        comments = sorted([row[0] for row in cursor])
+        comments = sorted(n for n, in self.env.db_query("""
+            SELECT number FROM fullblog_comments WHERE name=%s
+            """, (self.post_name,)))
         if comments:
             return comments[-1] + 1 # Add 1 for next free
         # No item found - need to double-check to find out why
@@ -407,8 +323,9 @@ class BlogPost(object):
 
     def __init__(self, env, name, version=0):
         self.env = env
+        self.log = env.log
         # Expand the default values as object properties
-        for prop in self._db_default_fields.keys():
+        for prop in self._db_default_fields:
             if isinstance(self._db_default_fields[prop], datetime.datetime):
                 # Default will evaluate to initial loading of the class itself
                 setattr(self, prop, datetime.datetime.now(utc))
@@ -430,6 +347,8 @@ class BlogPost(object):
         if not version_author:
             warnings.append(('version_author', 'Version author missing'))
         for attr in ['name', 'title', 'body', 'author']:
+            # TODO: use proper labels in the warnings. 'name' is the short name in
+            #       reality. This may be confusing.
             if not getattr(self, attr):
                 warnings.append((attr, '%s is empty.' % attr.capitalize()))
         # Return if initial problems, or only verification was wanted
@@ -440,22 +359,17 @@ class BlogPost(object):
         version = 1
         if self.versions:
             version = self.versions[-1] + 1
-        self.env.log.debug("Saving new version %d of blog post %r "
-                "from author %r" % (version, self.name, version_author))
-        sql = "INSERT INTO fullblog_posts " \
-                "(name, version, title, body, publish_time, version_time, " \
-                "version_comment, version_author, author, categories) " \
-                "VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s)"
-        args = (self.name, version, self.title, self.body,
-                to_timestamp(self.publish_time), version_time,
-                version_comment, version_author, self.author, self.categories)
-        if hasattr(self.env, 'db_transaction'):
-            self.env.db_transaction(sql, args)
-        else:
-            db = self.env.get_db_cnx()
-            cursor = db.cursor()
-            cursor.execute(sql, args)
-            db.commit()
+        self.log.debug("Saving new version %d of blog post %r "
+                       "from author %r", version, self.name, version_author)
+        self.env.db_transaction("""
+            INSERT INTO fullblog_posts
+            (name, version, title, body, publish_time, version_time,
+             version_comment, version_author, author, categories)
+            VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s)
+            """, (self.name, version, self.title, self.body,
+                  to_timestamp(self.publish_time), version_time,
+                  version_comment, version_author, self.author,
+                  self.categories))
         self._load_post(version)
         return warnings
 
@@ -465,7 +379,7 @@ class BlogPost(object):
         If no field values have actually changed it will return False, or
         True if one or more fields where updated. """
         changes_made = False
-        for field in fields.keys():
+        for field in fields:
             if not hasattr(self, field):
                 continue    # skip non-existing attributes
             if field in ['name', 'version']:
@@ -488,73 +402,53 @@ class BlogPost(object):
         else:
             sql = "DELETE FROM fullblog_posts WHERE name=%s"
             args = (self.name,)
-        if hasattr(self.env, 'db_transaction'):
-            self.env.db_transaction(sql, args)
-            db = None
-        else:
-            db = self.env.get_db_cnx()
-            cursor = db.cursor()
-            cursor.execute(sql, args)
-            db.commit()
-        if not len(self.get_versions()):
-            # Delete comments
-            for comment in self.get_comments():
-                comment.delete()
-            # Delete attachments
-            if db is not None:
-                Attachment.delete_all(self.env, 'blog', self.name, db)
-            else:
+        with self.env.db_transaction as db:
+            db(sql, args)
+            if not len(self.get_versions()):
+                # Delete comments
+                for comment in self.get_comments():
+                    comment.delete()
                 Attachment.delete_all(self.env, 'blog', self.name)
         return True
 
     def get_versions(self):
         """ Returns a sorted list of versions stored for the blog post.
         Returns empty list ([]) if no versions exists. """
-        sql = "SELECT version from fullblog_posts WHERE name=%s"
-        args = (self.name,)
-        if hasattr(self.env, 'db_query'):
-            cursor = self.env.db_query(sql, args)
-        else:
-            db = self.env.get_db_cnx()
-            cursor = db.cursor()
-            cursor.execute(sql, args)
-        self.versions = sorted([row[0] for row in cursor])
-        return self.versions
+        return [v for v, in self.env.db_query("""
+            SELECT version from fullblog_posts WHERE name=%s
+            """, (self.name,))]
 
     def get_comments(self):
-        """ Returns a list of used comment numbers attached to the post.
+        """Returns a list of used comment numbers attached to the post.
         It instantiates BlogComment objects for comments attached to the
-        current BlogPost, and returns them in a list sorted by number. """
+        current BlogPost, and returns them in a list sorted by number.
+        """
         comments = sorted(get_blog_comments(self.env, post_name=self.name),
-                    key=itemgetter(1))
+                    key=operator.itemgetter(1))
         return [BlogComment(self.env, comment[0],
                         comment[1]) for comment in comments]
 
     # Internal methods
 
     def _fetch_fields(self, version=0):
-        """ Returns a dict with field/value combinations for the content
+        """Returns a dict with field/value combinations for the content
         of a specific version of a blog post, or last/current version if
         version is 0.
-        Returns emtpy dict if no such post or post/version exists. """
+
+        Returns emtpy dict if no such post or post/version exists.
+        """
         self.versions = self.get_versions()
         if not self.versions or (version and not version in self.versions):
             # No blog post with the name exists
             return {}
         version = version or self.versions[-1]
-        sql = "SELECT title, body, publish_time, version_time, " \
-              "version_comment, version_author, author, categories " \
-              "FROM fullblog_posts " \
-              "WHERE name=%s AND version=%s"
-        args = (self.name, version)
-        if hasattr(self.env, 'db_query'):
-            cursor = self.env.db_query(sql, args)
-        else:
-            db = self.env.get_db_cnx()
-            cursor = db.cursor()
-            cursor.execute(sql, args)
         fields = {}
-        for row in cursor:
+        for row in self.env.db_query("""
+                SELECT title,body,publish_time,version_time,version_comment,
+                 version_author,author,categories
+                FROM fullblog_posts
+                WHERE name=%s AND version=%s
+                """, (self.name, version)):
             fields['version'] = version
             fields['title'] = row[0]
             fields['body'] = row[1]
