@@ -8,13 +8,11 @@ License: BSD
 (c) 2007 ::: www.CodeResort.com - BV Network AS (simon-code@bvnetwork.no)
 """
 
-# Imports from standard lib
 import datetime
 import re
+import operator
 from pkg_resources import resource_filename
 
-# Trac and Genshi imports
-from genshi.builder import tag
 from trac.attachment import AttachmentModule
 from trac.config import ListOption, BoolOption, IntOption
 from trac.core import *
@@ -23,32 +21,18 @@ from trac.search.api import ISearchSource, shorten_result
 from trac.timeline.api import ITimelineEventProvider
 from trac.util import arity
 from trac.util.datefmt import utc
+from trac.util.html import tag
 from trac.util.text import shorten_line
 from trac.util.translation import _
 from trac.web.api import IRequestHandler, HTTPNotFound
-from trac.web.chrome import INavigationContributor, ITemplateProvider, \
-        add_stylesheet, add_link, add_warning, add_notice, add_ctxtnav, prevnext_nav
-from trac.wiki.formatter import format_to
+from trac.web.chrome import Chrome, INavigationContributor, \
+        ITemplateProvider, add_stylesheet, add_link, add_warning, add_notice, \
+        add_ctxtnav, prevnext_nav, web_context
+from trac.wiki.formatter import format_to_oneliner
 
-try:
-    from trac.web.chrome import web_context          # Trac ~+1.3
-except ImportError:
-    from trac.mimeview.api import Context
-    web_context = Context.from_request
-
-try:
-    from trac.util.compat import itemgetter
-    from trac.util.compat import sorted, set
-except ImportError:
-    # 0.12 compat - sorted and set should already be part of Python 2.4
-    from operator import itemgetter
-
-# Imports from same package
-from model import *
-from core import FullBlogCore
-from util import map_month_names, parse_period
-
-__all__ = ['FullBlogModule']
+from .model import *
+from .core import FullBlogCore
+from .util import map_month_names, parse_period
 
 
 class FullBlogModule(Component):
@@ -57,22 +41,23 @@ class FullBlogModule(Component):
                ISearchSource, ITimelineEventProvider,
                ITemplateProvider)
 
-    # Options
-
     month_names = ListOption('fullblog', 'month_names',
-        doc = """Ability to specify a list of month names for display in groupings.
-        If empty it will make a list from default locale setting.
-        Enter list of 12 months like:
-        `month_names = January, February, ..., December` """)
+        doc="""Ability to specify a list of month names for display
+        in groupings. If empty it will make a list from default locale
+        setting.  Enter list of 12 months like:
+        `month_names = January, February, ..., December`
+        """)
 
     personal_blog = BoolOption('fullblog', 'personal_blog', False,
-        """When using the Blog as a personal blog (only one author), setting to 'True'
-        will disable the display of 'Browse by author:' in sidebar, and also removes
-        various author links and references. """)
+        """When using the Blog as a personal blog (only one author),
+        setting to 'True' will disable the display of 'Browse by author:'
+        in sidebar, and also removes various author links and references.
+        """)
 
     num_items = IntOption('fullblog', 'num_items_front', 20,
         """Option to specify how many recent posts to display on the
-        front page of the Blog (and RSS feeds).""")
+        front page of the Blog (and RSS feeds).
+        """)
 
     archive_rss_icon = BoolOption('fullblog', 'archive_rss_icon', True,
         """Controls whether or not to display the rss icon""")
@@ -120,10 +105,7 @@ class FullBlogModule(Component):
 
         command, pagename, path_items, listing_data = self._parse_path(req)
         action = req.args.get('action', 'view').lower()
-        try:
-            version = int(req.args.get('version', 0))
-        except:
-            version = 0
+        version = req.args.getint('version', 0)
 
         data = {}
         template = 'fullblog_view.html'
@@ -132,9 +114,8 @@ class FullBlogModule(Component):
         blog_month_names = map_month_names(
                     self.env.config.getlist('fullblog', 'month_names'))
         data['blog_month_names'] = blog_month_names
-        self.env.log.debug(
-            "Blog debug: command=%r, pagename=%r, path_items=%r" % (
-                command, pagename, path_items))
+        self.log.debug("Blog debug: command=%r, pagename=%r, path_items=%r",
+                       command, pagename, path_items)
 
         if not command:
             # Request for just root (display latest)
@@ -190,6 +171,7 @@ class FullBlogModule(Component):
                 if 'cancelcomment' in req.args:
                     req.redirect(req.href.blog(pagename))
                 elif 'previewcomment' in req.args:
+                    data['comment_preview'] = True
                     warnings.extend(blog_core.create_comment(req, comment, verify_only=True))
                 elif 'submitcomment' in req.args and not warnings:
                     warnings.extend(blog_core.create_comment(req, comment))
@@ -302,7 +284,7 @@ class FullBlogModule(Component):
             req.perm(bp.resource).require('BLOG_DELETE')
             if 'blog-cancel' in req.args:
                 req.redirect(req.href.blog(pagename))
-            comment = int(req.args.get('comment', '0'))
+            comment = req.args.getint('comment', 0)
             warnings = []
             if comment:
                 # Deleting a specific comment
@@ -324,7 +306,7 @@ class FullBlogModule(Component):
                     raise TracError(
                             "Cannot delete. Blog post '%s' does not exist." % (
                                     bp.name))
-                version = int(req.args.get('version', '0'))
+                version = req.args.getint('version', 0)
                 if req.method == 'POST':
                     if 'blog-version-delete' in req.args:
                         if bp.version != version:
@@ -399,6 +381,10 @@ class FullBlogModule(Component):
         if 'BLOG_CREATE' in req.perm('blog'):
             add_ctxtnav(req, 'New Post', href=req.href.blog('create'),
                     title="Create new Blog Post")
+
+        chrome = Chrome(self.env)
+        chrome.add_auto_preview(req)
+        chrome.add_wiki_toolbars(req)
         add_stylesheet(req, 'tracfullblog/css/fullblog.css')
         add_stylesheet(req, 'common/css/code.css')
         data['blog_personal_blog'] = self.env.config.getbool('fullblog',
@@ -406,7 +392,7 @@ class FullBlogModule(Component):
         data['blog_archive_rss_icon'] = self.all_rss_icons \
                                         or self.archive_rss_icon
         data['blog_all_rss_icons'] = self.all_rss_icons
-        return (template, data, None)
+        return template, data, {}
 
     # ISearchSource methods
 
@@ -419,7 +405,7 @@ class FullBlogModule(Component):
         is searchable by default.
         """
         if 'BLOG_VIEW' in req.perm('blog', id=None):
-            yield ('blog', 'Blog')
+            yield 'blog', 'Blog'
 
     def get_search_results(self, req, terms, filters):
         """Return a list of search results matching each search term in `terms`.
@@ -458,8 +444,8 @@ class FullBlogModule(Component):
     # ITimelineEventProvider methods
 
     def get_timeline_filters(self, req):
-        if 'BLOG_VIEW' in req.perm('blog', id=None):
-            yield ('blog', _('Blog details'))
+        if 'BLOG_VIEW' in req.perm('blog'):
+            yield 'blog', _("Blog details")
 
     def get_timeline_events(self, req, start, stop, filters):
         if 'blog' in filters:
@@ -484,30 +470,26 @@ class FullBlogModule(Component):
                 yield event
             # Blog comments
             blog_comments = get_blog_comments(self.env, from_dt=start, to_dt=stop)
-            blog_comments = sorted(blog_comments, key=itemgetter(4), reverse=True)
+            blog_comments = sorted(blog_comments, key=operator.itemgetter(4),
+                                   reverse=True)
             for post_name, number, comment, author, time in blog_comments:
                 bp_resource = blog_realm(id=post_name)
                 if 'BLOG_VIEW' not in req.perm(bp_resource):
                     continue
                 bp = BlogPost(self.env, post_name)
                 bc = BlogComment(self.env, post_name, number=number)
-                yield ('blog', time, author, (bp_resource, bp, bc))
+                yield 'blog', time, author, (bp_resource, bp, bc)
 
     def render_timeline_event(self, context, field, event):
         bp_resource, bp, bc = event[3]
-        compat_format_0_11_2 = 'oneliner'
-        if hasattr(context, '_hints'):
-            compat_format_0_11_2 = None
         if bc: # A blog comment
             if field == 'url':
                 return context.href.blog(bp.name) + '#comment-%d' % bc.number
             elif field == 'title':
                 return tag('Blog: ', tag.em(bp.title), ' comment added')
             elif field == 'description':
-                comment = compat_format_0_11_2 and shorten_line(bc.comment) \
-                            or bc.comment
-                return format_to(self.env, compat_format_0_11_2,
-                            context(resource=bp_resource), comment)
+                return format_to_oneliner(
+                        self.env, context(resource=bp_resource), bc.comment)
         else: # A blog post
             if field == 'url':
                 return context.href.blog(bp.name)
@@ -515,26 +497,24 @@ class FullBlogModule(Component):
                 return tag('Blog: ', tag.em(bp.title),
                         bp.version > 1 and ' edited' or ' created')
             elif field == 'description':
-                comment = compat_format_0_11_2 and shorten_line(bp.version_comment) \
-                            or bp.version_comment
-                return format_to(self.env, compat_format_0_11_2,
-                        context(resource=bp_resource), comment)
+                return format_to_oneliner(
+                        self.env, context(resource=bp_resource),
+                        bp.version_comment)
 
     # ITemplateProvider methods
 
     def get_htdocs_dirs(self):
-        """ Makes the 'htdocs' folder inside the egg available. """
         return [('tracfullblog', resource_filename('tracfullblog', 'htdocs'))]
 
     def get_templates_dirs(self):
-        """ Location of Trac templates provided by plugin. """
         return [resource_filename('tracfullblog', 'templates')]
 
     # Internal methods
 
     def _parse_path(self, req):
-        """ Parses the request path for the blog and returns a
-        ('command', 'pagename', 'path_items', 'listing_data') tuple. """
+        """Parses the request path for the blog and returns a
+        ('command', 'pagename', 'path_items', 'listing_data') tuple.
+        """
         # Parse out the path and actions from args
         path = req.args.get('blog_path', '')
         path_items = path.split('/')
@@ -543,17 +523,19 @@ class FullBlogModule(Component):
         listing_data = {}
         from_dt, to_dt = parse_period(path_items)
         if not path_items:
-            pass # emtpy default for return is fine
-        elif len(path_items) > 1 and path_items[0].lower() in ['view', 'edit', 'delete']:
+            pass # empty default for return is fine
+        elif len(path_items) > 1 and \
+                path_items[0].lower() in ['view', 'edit', 'delete']:
             command = path_items[0].lower()
             pagename = '/'.join(path_items[1:])
         elif len(path_items) == 1 and path_items[0].lower() == 'archive':
             command = path_items[0].lower()
         elif len(path_items) >= 1 and path_items[0].lower() == 'create':
             command = path_items[0].lower()
-            pagename = req.args.get('name','') or (len(path_items) > 1 \
-                                                    and '/'.join(path_items[1:]))
-        elif len(path_items) > 1 and path_items[0].lower() in ['author', 'category']:
+            pagename = req.args.get('name','') or \
+                    (len(path_items) > 1 and '/'.join(path_items[1:]))
+        elif len(path_items) > 1 and \
+                path_items[0].lower() in ['author', 'category']:
             command = 'listing' + '-' + path_items[0].lower()
             listing_data[path_items[0].lower()] = '/'.join(path_items[1:])
         elif len(path_items) == 2 and (from_dt, to_dt) != (None, None):
@@ -564,4 +546,4 @@ class FullBlogModule(Component):
             # A request for a regular page
             command = 'view'
             pagename = path
-        return (command, pagename, path_items, listing_data)
+        return command, pagename, path_items, listing_data
